#define _GNU_SOURCE

#include <syslog.h>
#include <rpc/rpc.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <dlfcn.h>
#include <stdlib.h>

SVCXPRT*
svcudp_create(int sock)
{
  static SVCXPRT* (*real_svcudp_create)(int);
  static int port = 0;

  if (!real_svcudp_create) {
    real_svcudp_create = dlsym(RTLD_NEXT, "svcudp_create");
    if (!real_svcudp_create)
      abort();
    char *env_var = getenv("LIBSVCUDP_CREATE_BINDPORT");
    if (env_var) {
      char *ptr;
      errno = 0;
      int tentative_port = strtol(env_var, &ptr, 0);
      if (errno == 0 && *ptr == '\0'
	  && tentative_port > 0 && tentative_port < (1<<16))
	port = tentative_port;
    }
  }

  if (port != 0 && sock == RPC_ANYSOCK) {
    sock = socket(SOCK_DGRAM, AF_INET, IPPROTO_UDP);
    if (sock < 0) {
      syslog(LOG_ERR, "libsvcudp_create_bindport: socket(): %m");
      exit(1);
    }
    struct sockaddr_in sin;
    memset(&sin, 0, sizeof(sin));
    sin.sin_family = AF_INET;
    sin.sin_addr.s_addr = htonl(INADDR_ANY);
    sin.sin_port   = htons(port);
    if (bind(sock, &sin, sizeof(sin)) < 0) {
      syslog(LOG_ERR, "libsvcudp_create_bindport: bind(port=%d): %m", port);
      exit(1);
    }
  }

  return real_svcudp_create(sock);
}

/*
  Local Variables:
  compile-command: "gcc -g -Wall -Werror -fPIC -shared -I/usr/include/tirpc -o libsvcudp_create_bindport.so svcudp_create_bindport.c"
  End:
 */
